class Action:

    Engine_Actor_RemoteRole = -1
    Engine_Actor_DrawScale = -1
    TAGame_CarComponent_TA_Vehicle = -1
    TAGame_CarComponent_TA_ReplicatedActive = -1
    TAGame_CarComponent_Boost_TA_ReplicatedBoostAmount = -1
    TAGame_GameEvent_TA_ReplicatedRoundCountDownNumber = -1
    TAGame_GameEvent_TA_ReplicatedGameStateTimeRemaining = -1
    TAGame_GameEvent_TA_ReplicatedStateName = -1
    TAGame_GameEvent_TA_BotSkill = -1
    TAGame_GameEvent_TA_bHasLeaveMatchPenalty = -1
    TAGame_GameEvent_Team_TA_MaxTeamSize = -1
    TAGame_GameEvent_Soccar_TA_RoundNum = -1
    TAGame_GameEvent_Soccar_TA_bBallHasBeenHit = -1
    TAGame_GameEvent_Soccar_TA_SecondsRemaining = -1
    Engine_Pawn_PlayerReplicationInfo = -1
    TAGame_RBActor_TA_ReplicatedRBState = -1
    TAGame_Vehicle_TA_ReplicatedSteer = -1
    TAGame_Vehicle_TA_ReplicatedThrottle = -1
    TAGame_Vehicle_TA_bReplicatedHandbrake = -1
    TAGame_Vehicle_TA_bDriving = -1
    TAGame_Car_TA_RumblePickups = -1
    TAGame_Car_TA_TeamPaint = -1
    TAGame_CameraSettingsActor_TA_CameraYaw = -1
    TAGame_CameraSettingsActor_TA_CameraPitch = -1
    TAGame_CameraSettingsActor_TA_bMouseCameraToggleEnabled = -1
    TAGame_CameraSettingsActor_TA_bUsingSwivel = -1
    TAGame_CameraSettingsActor_TA_bUsingBehindView = -1
    TAGame_CameraSettingsActor_TA_ProfileSettings = -1
    TAGame_CameraSettingsActor_TA_bUsingSecondaryCamera = -1
    Engine_PlayerReplicationInfo_UniqueId = -1
    Engine_PlayerReplicationInfo_Team = -1
    Engine_PlayerReplicationInfo_PlayerID = -1
    Engine_PlayerReplicationInfo_PlayerName = -1
    Engine_PlayerReplicationInfo_Ping = -1
    TAGame_PRI_TA_CurrentVoiceRoom = -1
    TAGame_PRI_TA_SpectatorShortcut = -1
    TAGame_PRI_TA_SteeringSensitivity = -1
    TAGame_PRI_TA_Title = -1
    TAGame_PRI_TA_PartyLeader = -1
    TAGame_PRI_TA_ClientLoadoutsOnline = -1
    TAGame_PRI_TA_ClientLoadouts = -1
    TAGame_PRI_TA_ReplicatedGameEvent = -1
    TAGame_PRI_TA_PlayerHistoryValid = -1
    TAGame_PRI_TA_MatchScore = -1
    TAGame_Ball_TA_HitTeamNum = -1
    TAGame_CarComponent_Dodge_TA_DodgeTorque = -1
    Engine_GameReplicationInfo_ServerName = -1
    ProjectX_GRI_X_MatchGuid = -1
    ProjectX_GRI_X_bGameStarted = -1
    ProjectX_GRI_X_GameServerID = -1
    ProjectX_GRI_X_Reservations = -1
    ProjectX_GRI_X_ReplicatedServerRegion = -1
    ProjectX_GRI_X_ReplicatedGamePlaylist = -1
    TAGame_Team_TA_GameEvent = -1
    TAGame_VehiclePickup_TA_NewReplicatedPickupData = -1
    TAGame_PRI_TA_PersistentCamera = -1
    TAGame_CameraSettingsActor_TA_PRI = -1
    TAGame_Ball_TA_GameEvent = -1
    TAGame_PRI_TA_MatchGoals = -1
    TAGame_PRI_TA_MatchAssists = -1
    TAGame_PRI_TA_MatchSaves = -1
    TAGame_PRI_TA_MatchShots = -1
    Engine_PlayerReplicationInfo_Score = -1
    Engine_TeamInfo_Score = -1
    TAGame_GameEvent_TA_bCanVoteToForfeit = -1
    TAGame_Car_TA_ReplicatedDemolishGoalExplosion = -1
    TAGame_GameEvent_Soccar_TA_ReplicatedScoredOnTeam = -1
    Engine_Actor_bHidden = -1
    TAGame_PRI_TA_bReady = -1
    TAGame_PRI_TA_ReplicatedWorstNetQualityBeyondLatency = -1
    TAGame_CarComponent_FlipCar_TA_FlipCarTime = -1
    TAGame_GameEvent_Soccar_TA_MaxScore = -1
    TAGame_Team_TA_ClubColors = -1
    TAGame_Car_TA_ClubColors = -1
    ProjectX_GRI_X_ReplicatedGameMutatorIndex = -1
    TAGame_CarComponent_FlipCar_TA_bFlipRight = -1
    TAGame_PRI_TA_bIsDistracted = -1
    TAGame_Team_TA_Difficulty = -1
    TAGame_Team_TA_CustomTeamName = -1
    ProjectX_GRI_X_MatchGUID = -1
    TAGame_GameEvent_Soccar_TA_SeriesLength = -1
    TAGame_CarComponent_Dodge_TA_DodgeImpulse = -1
    Engine_PlayerReplicationInfo_RemoteUserData = -1
    TAGame_GameEvent_Soccar_TA_bOverTime = -1
    TAGame_GameEvent_Soccar_TA_bClubMatch = -1
    TAGame_Team_TA_ClubID = -1
    Engine_PlayerReplicationInfo_bBot = -1
    TAGame_PRI_TA_BotProductName = -1
    TAGame_PRI_TA_ClubID = -1
    TAGame_GameEvent_Team_TA_bForfeit = -1
    TAGame_PRI_TA_PlayerHistoryKey = -1
    Engine_PlayerReplicationInfo_bTimedOut = -1
    Engine_ReplicatedActor_ORS_ReplicatedOwner = -1
    TAGame_MaxTimeWarningData_TA_EndGameWarningEpochTime = -1
    TAGame_MaxTimeWarningData_TA_EndGameEpochTime = -1
    TAGame_Ball_TA_ReplicatedWorldBounceScale = -1
    TAGame_PRI_TA_bIsInSplitScreen = -1
    TAGame_RBActor_TA_bReplayActor = -1
    TAGame_GameEvent_Soccar_TA_bUnlimitedTime = -1
    TAGame_VehiclePickup_TA_bNoPickup = -1
    TAGame_CarComponent_Boost_TA_UnlimitedBoostRefCount = -1
    TAGame_PRI_TA_PawnType = -1
    TAGame_CarComponent_Boost_TA_RechargeDelay = -1
    TAGame_CarComponent_Boost_TA_RechargeRate = -1
    TAGame_BreakOutActor_Platform_TA_DamageState = -1
    TAGame_Ball_Breakout_TA_LastTeamTouch = -1
    TAGame_PRI_TA_MatchBreakoutDamage = -1
    TAGame_Ball_Breakout_TA_AppliedDamage = -1
    TAGame_Ball_Breakout_TA_DamageIndex = -1
    TAGame_PRI_TA_bUsingItems = -1
    TAGame_RumblePickups_TA_ConcurrentItemCount = -1
    TAGame_RumblePickups_TA_PickupInfo = -1
    TAGame_CarComponent_TA_ReplicatedActivityTime = -1
    TAGame_SpecialPickup_Targeted_TA_Targeted = -1
    TAGame_SpecialPickup_BallFreeze_TA_RepOrigSpeed = -1
    TAGame_Car_TA_AddedCarForceMultiplier = -1
    TAGame_Car_TA_AddedBallForceMultiplier = -1
    TAGame_Car_TA_AddedBallForceMultiplier = -1
    TAGame_Team_Soccar_TA_GameScore = -1
    TAGame_RBActor_TA_bIgnoreSyncing = -1
    TAGame_RBActor_TA_WeldedInfo = -1
    TAGame_SpecialPickup_BallVelcro_TA_AttachTime = -1
    TAGame_SpecialPickup_BallVelcro_TA_bHit = -1
    TAGame_SpecialPickup_BallVelcro_TA_BreakTime = -1
    TAGame_SpecialPickup_BallVelcro_TA_bBroken = -1
    TAGame_Car_TA_ReplicatedDemolish_CustomFX = -1
    TAGame_Car_TA_ReplicatedDemolish = -1
    
    Archetypes_Car_Car_Default = -1 # car
    Archetypes_Ball_Ball_Default = -1 # ball
    TAGame_Default__PRI_TA = -1 # player
    Archetypes_GameEvent_GameEvent_Hockey = -1 # hockey puck
    Archetypes_GameEvent_GameEvent_Basketball = -1 # basketball (hoops ball)
    
    TAGame_GameEvent_TA_MatchTypeClass = -1 # weird event
    Engine_GameReplicationInfo_GameClass = -1 # weird event
    TAGame_GameEvent_Soccar_TA_ReplicatedStatEvent = -1 # weird event
    TAGame_GameEvent_Soccar_TA_SubRulesArchetype = -1 # weird event


    @staticmethod
    def set_values(objects):

        # reset all values
        # if we don't do this, we'll get a lot of false positives
        Action.reset()

        for i, obj in enumerate(objects):
            match obj:
                case 'Engine.Actor:RemoteRole':
                    Action.Engine_Actor_RemoteRole = i
                case 'Engine.Actor:DrawScale':
                    Action.Engine_Actor_DrawScale = i
                case 'TAGame.CarComponent_TA:Vehicle':
                    Action.TAGame_CarComponent_TA_Vehicle = i
                case 'TAGame.CarComponent_TA:ReplicatedActive':
                    Action.TAGame_CarComponent_TA_ReplicatedActive = i
                case 'TAGame.CarComponent_Boost_TA:ReplicatedBoostAmount':
                    Action.TAGame_CarComponent_Boost_TA_ReplicatedBoostAmount = i
                case 'TAGame.GameEvent_TA:ReplicatedRoundCountDownNumber':
                    Action.TAGame_GameEvent_TA_ReplicatedRoundCountDownNumber = i
                case 'TAGame.GameEvent_TA:ReplicatedGameStateTimeRemaining':
                    Action.TAGame_GameEvent_TA_ReplicatedGameStateTimeRemaining = i
                case 'TAGame.GameEvent_TA:ReplicatedStateName':
                    Action.TAGame_GameEvent_TA_ReplicatedStateName = i
                case 'TAGame.GameEvent_TA:BotSkill':
                    Action.TAGame_GameEvent_TA_BotSkill = i
                case 'TAGame.GameEvent_TA:bHasLeaveMatchPenalty':
                    Action.TAGame_GameEvent_TA_bHasLeaveMatchPenalty = i
                case 'TAGame.GameEvent_Team_TA:MaxTeamSize':
                    Action.TAGame_GameEvent_Team_TA_MaxTeamSize = i
                case 'TAGame.GameEvent_Soccar_TA:RoundNum':
                    Action.TAGame_GameEvent_Soccar_TA_RoundNum = i
                case 'TAGame.GameEvent_Soccar_TA:bBallHasBeenHit':
                    Action.TAGame_GameEvent_Soccar_TA_bBallHasBeenHit = i
                case 'TAGame.GameEvent_Soccar_TA:SecondsRemaining':
                    Action.TAGame_GameEvent_Soccar_TA_SecondsRemaining = i
                case 'Engine.Pawn:PlayerReplicationInfo':
                    Action.Engine_Pawn_PlayerReplicationInfo = i
                case 'TAGame.RBActor_TA:ReplicatedRBState':
                    Action.TAGame_RBActor_TA_ReplicatedRBState = i
                case 'TAGame.Vehicle_TA:ReplicatedSteer':
                    Action.TAGame_Vehicle_TA_ReplicatedSteer = i
                case 'TAGame.Vehicle_TA:ReplicatedThrottle':
                    Action.TAGame_Vehicle_TA_ReplicatedThrottle = i
                case 'TAGame.Vehicle_TA:bReplicatedHandbrake':
                    Action.TAGame_Vehicle_TA_bReplicatedHandbrake = i
                case 'TAGame.Vehicle_TA:bDriving':
                    Action.TAGame_Vehicle_TA_bDriving = i
                case 'TAGame.Car_TA:RumblePickups':
                    Action.TAGame_Car_TA_RumblePickups = i
                case 'TAGame.Car_TA:TeamPaint':
                    Action.TAGame_Car_TA_TeamPaint = i
                case 'TAGame.CameraSettingsActor_TA:CameraYaw':
                    Action.TAGame_CameraSettingsActor_TA_CameraYaw = i
                case 'TAGame.CameraSettingsActor_TA:CameraPitch':
                    Action.TAGame_CameraSettingsActor_TA_CameraPitch = i
                case 'TAGame.CameraSettingsActor_TA:bMouseCameraToggleEnabled':
                    Action.TAGame_CameraSettingsActor_TA_bMouseCameraToggleEnabled = i
                case 'TAGame.CameraSettingsActor_TA:bUsingSwivel':
                    Action.TAGame_CameraSettingsActor_TA_bUsingSwivel = i
                case 'TAGame.CameraSettingsActor_TA:bUsingBehindView':
                    Action.TAGame_CameraSettingsActor_TA_bUsingBehindView = i
                case 'TAGame.CameraSettingsActor_TA:ProfileSettings':
                    Action.TAGame_CameraSettingsActor_TA_ProfileSettings = i
                case 'TAGame.CameraSettingsActor_TA:bUsingSecondaryCamera':
                    Action.TAGame_CameraSettingsActor_TA_bUsingSecondaryCamera = i
                case 'Engine.PlayerReplicationInfo:UniqueId':
                    Action.Engine_PlayerReplicationInfo_UniqueId = i
                case 'Engine.PlayerReplicationInfo:Team':
                    Action.Engine_PlayerReplicationInfo_Team = i
                case 'Engine.PlayerReplicationInfo:PlayerID':
                    Action.Engine_PlayerReplicationInfo_PlayerID = i
                case 'Engine.PlayerReplicationInfo:PlayerName':
                    Action.Engine_PlayerReplicationInfo_PlayerName = i
                case 'Engine.PlayerReplicationInfo:Ping':
                    Action.Engine_PlayerReplicationInfo_Ping = i
                case 'TAGame.PRI_TA:CurrentVoiceRoom':
                    Action.TAGame_PRI_TA_CurrentVoiceRoom = i
                case 'TAGame.PRI_TA:SpectatorShortcut':
                    Action.TAGame_PRI_TA_SpectatorShortcut = i
                case 'TAGame.PRI_TA:SteeringSensitivity':
                    Action.TAGame_PRI_TA_SteeringSensitivity = i
                case 'TAGame.PRI_TA:Title':
                    Action.TAGame_PRI_TA_Title = i
                case 'TAGame.PRI_TA:PartyLeader':
                    Action.TAGame_PRI_TA_PartyLeader = i
                case 'TAGame.PRI_TA:ClientLoadoutsOnline':
                    Action.TAGame_PRI_TA_ClientLoadoutsOnline = i
                case 'TAGame.PRI_TA:ClientLoadouts':
                    Action.TAGame_PRI_TA_ClientLoadouts = i
                case 'TAGame.PRI_TA:ReplicatedGameEvent':
                    Action.TAGame_PRI_TA_ReplicatedGameEvent = i
                case 'TAGame.PRI_TA:PlayerHistoryValid':
                    Action.TAGame_PRI_TA_PlayerHistoryValid = i
                case 'TAGame.PRI_TA:MatchScore':
                    Action.TAGame_PRI_TA_MatchScore = i
                case 'TAGame.Ball_TA:HitTeamNum':
                    Action.TAGame_Ball_TA_HitTeamNum = i
                case 'TAGame.CarComponent_Dodge_TA:DodgeTorque':
                    Action.TAGame_CarComponent_Dodge_TA_DodgeTorque = i
                case 'Engine.GameReplicationInfo:ServerName':
                    Action.Engine_GameReplicationInfo_ServerName = i
                case 'ProjectX.GRI_X:MatchGuid':
                    Action.ProjectX_GRI_X_MatchGuid = i
                case 'ProjectX.GRI_X:bGameStarted':
                    Action.ProjectX_GRI_X_bGameStarted = i
                case 'ProjectX.GRI_X:GameServerID':
                    Action.ProjectX_GRI_X_GameServerID = i
                case 'ProjectX.GRI_X:Reservations':
                    Action.ProjectX_GRI_X_Reservations = i
                case 'ProjectX.GRI_X:ReplicatedServerRegion':
                    Action.ProjectX_GRI_X_ReplicatedServerRegion = i
                case 'ProjectX.GRI_X:ReplicatedGamePlaylist':
                    Action.ProjectX_GRI_X_ReplicatedGamePlaylist = i
                case 'TAGame.Team_TA:GameEvent':
                    Action.TAGame_Team_TA_GameEvent = i
                case 'TAGame.VehiclePickup_TA:NewReplicatedPickupData':
                    Action.TAGame_VehiclePickup_TA_NewReplicatedPickupData = i
                case 'Archetypes.Car.Car_Default':
                    Action.Archetypes_Car_Car_Default = i
                case 'Archetypes.Ball.Ball_Default':
                    Action.Archetypes_Ball_Ball_Default = i
                case 'TAGame.Default__PRI_TA':
                    Action.TAGame_Default__PRI_TA = i
                case 'Archetypes.GameEvent.GameEvent_Soccar':
                    Action.Archetypes_GameEvent_GameEvent_Soccar = i
                case 'TAGame.PRI_TA:PersistentCamera':
                    Action.TAGame_PRI_TA_PersistentCamera = i
                case 'TAGame.CameraSettingsActor_TA:PRI':
                    Action.TAGame_CameraSettingsActor_TA_PRI = i
                case 'TAGame.Ball_TA:GameEvent':
                    Action.TAGame_Ball_TA_GameEvent = i
                case 'TAGame.PRI_TA:MatchGoals':
                    Action.TAGame_PRI_TA_MatchGoals = i
                case 'TAGame.PRI_TA:MatchAssists':
                    Action.TAGame_PRI_TA_MatchAssists = i
                case 'TAGame.PRI_TA:MatchSaves':
                    Action.TAGame_PRI_TA_MatchSaves = i
                case 'TAGame.PRI_TA:MatchShots':
                    Action.TAGame_PRI_TA_MatchShots = i
                case 'Engine.PlayerReplicationInfo:Score':
                    Action.Engine_PlayerReplicationInfo_Score = i
                case 'Engine.TeamInfo:Score':
                    Action.Engine_TeamInfo_Score = i
                case 'TAGame.GameEvent_TA:bCanVoteToForfeit':
                    Action.TAGame_GameEvent_TA_bCanVoteToForfeit = i
                case 'TAGame.Car_TA:ReplicatedDemolishGoalExplosion':
                    Action.TAGame_Car_TA_ReplicatedDemolishGoalExplosion = i
                case 'TAGame.GameEvent_Soccar_TA:ReplicatedScoredOnTeam':
                    Action.TAGame_GameEvent_Soccar_TA_ReplicatedScoredOnTeam = i
                case 'Engine.Actor:bCollideActors':
                    Action.Engine_Actor_bCollideActors = i
                case 'Engine.Actor:bBlockActors':
                    Action.Engine_Actor_bBlockActors = i
                case 'TAGame.Ball_TA:ReplicatedExplosionDataExtended':
                    Action.TAGame_Ball_TA_ReplicatedExplosionDataExtended = i
                case 'Engine.Actor:bHidden':
                    Action.Engine_Actor_bHidden = i
                case 'TAGame.PRI_TA:bReady':
                    Action.TAGame_PRI_TA_bReady = i
                case 'TAGame.PRI_TA:ReplicatedWorstNetQualityBeyondLatency':
                    Action.TAGame_PRI_TA_ReplicatedWorstNetQualityBeyondLatency = i
                case 'TAGame.CarComponent_FlipCar_TA:FlipCarTime':
                    Action.TAGame_CarComponent_FlipCar_TA_FlipCarTime = i
                case 'TAGame.GameEvent_TA:MatchTypeClass':
                    Action.TAGame_GameEvent_TA_MatchTypeClass = i
                case 'Engine.GameReplicationInfo:GameClass':
                    Action.Engine_GameReplicationInfo_GameClass = i
                case 'TAGame.GameEvent_Soccar_TA:ReplicatedStatEvent':
                    Action.TAGame_GameEvent_Soccar_TA_ReplicatedStatEvent = i
                case 'TAGame.GameEvent_Soccar_TA:MaxScore':
                    Action.TAGame_GameEvent_Soccar_TA_MaxScore = i
                case 'TAGame.Team_TA:ClubColors':
                    Action.TAGame_Team_TA_ClubColors = i
                case 'TAGame.Car_TA:ClubColors':
                    Action.TAGame_Car_TA_ClubColors = i
                case 'ProjectX.GRI_X:ReplicatedGameMutatorIndex':
                    Action.ProjectX_GRI_X_ReplicatedGameMutatorIndex = i
                case 'TAGame.CarComponent_FlipCar_TA:bFlipRight':
                    Action.TAGame_CarComponent_FlipCar_TA_bFlipRight = i
                case 'TAGame.PRI_TA:bIsDistracted':
                    Action.TAGame_PRI_TA_bIsDistracted = i
                case 'TAGame.Team_TA:Difficulty':
                    Action.TAGame_Team_TA_Difficulty = i
                case 'TAGame.Team_TA:CustomTeamName':
                    Action.TAGame_Team_TA_CustomTeamName = i
                case 'ProjectX.GRI_X:MatchGUID':
                    Action.ProjectX_GRI_X_MatchGUID = i
                case 'TAGame.GameEvent_Soccar_TA:SeriesLength':
                    Action.TAGame_GameEvent_Soccar_TA_SeriesLength = i
                case 'TAGame.CarComponent_Dodge_TA:DodgeImpulse':
                    Action.TAGame_CarComponent_Dodge_TA_DodgeImpulse = i
                case 'Engine.PlayerReplicationInfo:RemoteUserData':
                    Action.Engine_PlayerReplicationInfo_RemoteUserData = i
                case 'TAGame.GameEvent_Soccar_TA:bOverTime':
                    Action.TAGame_GameEvent_Soccar_TA_bOverTime = i
                case 'TAGame.GameEvent_Soccar_TA:bClubMatch':
                    Action.TAGame_GameEvent_Soccar_TA_bClubMatch = i
                case 'TAGame.Team_TA:ClubID':
                    Action.TAGame_Team_TA_ClubID = i
                case 'Engine.PlayerReplicationInfo:bBot':
                    Action.Engine_PlayerReplicationInfo_bBot = i
                case 'TAGame.PRI_TA:BotProductName':
                    Action.TAGame_PRI_TA_BotProductName = i
                case 'TAGame.PRI_TA:ClubID':
                    Action.TAGame_PRI_TA_ClubID = i
                case 'TAGame.GameEvent_Team_TA:bForfeit':
                    Action.TAGame_GameEvent_Team_TA_bForfeit = i
                case 'TAGame.PRI_TA:PlayerHistoryKey':
                    Action.TAGame_PRI_TA_PlayerHistoryKey = i
                case 'Engine.PlayerReplicationInfo:bTimedOut':
                    Action.Engine_PlayerReplicationInfo_bTimedOut = i
                case 'Engine.ReplicatedActor_ORS:ReplicatedOwner':
                    Action.Engine_ReplicatedActor_ORS_ReplicatedOwner = i
                case 'TAGame.MaxTimeWarningData_TA:EndGameWarningEpochTime':
                    Action.TAGame_MaxTimeWarningData_TA_EndGameWarningEpochTime = i
                case 'TAGame.MaxTimeWarningData_TA:EndGameEpochTime':
                    Action.TAGame_MaxTimeWarningData_TA_EndGameEpochTime = i
                case 'Archetypes.GameEvent.GameEvent_Hockey':
                    Action.Archetypes_GameEvent_GameEvent_Hockey = i
                case 'TAGame.Ball_TA:ReplicatedWorldBounceScale':
                    Action.TAGame_Ball_TA_ReplicatedWorldBounceScale = i
                case 'TAGame.PRI_TA:bIsInSplitScreen':
                    Action.TAGame_PRI_TA_bIsInSplitScreen = i
                case 'TAGame.RBActor_TA:bReplayActor':
                    Action.TAGame_RBActor_TA_bReplayActor = i
                case 'Archetypes.GameEvent.GameEvent_Basketball':
                    Action.Archetypes_GameEvent_GameEvent_Basketball = i
                case 'TAGame.GameEvent_Soccar_TA:bUnlimitedTime':
                    Action.TAGame_GameEvent_Soccar_TA_bUnlimitedTime = i
                case 'TAGame.VehiclePickup_TA:bNoPickup':
                    Action.TAGame_VehiclePickup_TA_bNoPickup = i
                case 'TAGame.CarComponent_Boost_TA:UnlimitedBoostRefCount':
                    Action.TAGame_CarComponent_Boost_TA_UnlimitedBoostRefCount = i
                case 'TAGame.PRI_TA:PawnType':
                    Action.TAGame_PRI_TA_PawnType = i
                case 'TAGame.CarComponent_Boost_TA:RechargeDelay':
                    Action.TAGame_CarComponent_Boost_TA_RechargeDelay = i
                case 'TAGame.CarComponent_Boost_TA:RechargeRate':
                    Action.TAGame_CarComponent_Boost_TA_RechargeRate = i
                case 'TAGame.BreakOutActor_Platform_TA:DamageState':
                    Action.TAGame_BreakOutActor_Platform_TA_DamageState = i
                case 'TAGame.Ball_Breakout_TA:LastTeamTouch':
                    Action.TAGame_Ball_Breakout_TA_LastTeamTouch = i
                case 'TAGame.PRI_TA:MatchBreakoutDamage':
                    Action.TAGame_PRI_TA_MatchBreakoutDamage = i
                case 'TAGame.Ball_Breakout_TA:AppliedDamage':
                    Action.TAGame_Ball_Breakout_TA_AppliedDamage = i
                case 'TAGame.Ball_Breakout_TA:DamageIndex':
                    Action.TAGame_Ball_Breakout_TA_DamageIndex = i
                case 'TAGame.GameEvent_Soccar_TA:SubRulesArchetype':
                    Action.TAGame_GameEvent_Soccar_TA_SubRulesArchetype = i
                case 'TAGame.PRI_TA:bUsingItems':
                    Action.TAGame_PRI_TA_bUsingItems = i
                case 'TAGame.RumblePickups_TA:ConcurrentItemCount':
                    Action.TAGame_RumblePickups_TA_ConcurrentItemCount = i
                case 'TAGame.RumblePickups_TA:PickupInfo':
                    Action.TAGame_RumblePickups_TA_PickupInfo = i
                case 'TAGame.CarComponent_TA:ReplicatedActivityTime':
                    Action.TAGame_CarComponent_TA_ReplicatedActivityTime = i
                case 'TAGame.SpecialPickup_Targeted_TA:Targeted':
                    Action.TAGame_SpecialPickup_Targeted_TA_Targeted = i
                case 'TAGame.SpecialPickup_BallFreeze_TA:RepOrigSpeed':
                    Action.TAGame_SpecialPickup_BallFreeze_TA_RepOrigSpeed = i
                case 'TAGame.Car_TA:AddedCarForceMultiplier':
                    Action.TAGame_Car_TA_AddedCarForceMultiplier = i
                case 'TAGame.Car_TA:AddedBallForceMultiplier.':
                    Action.TAGame_Car_TA_AddedBallForceMultiplier = i
                case 'TAGame.Car_TA:AddedBallForceMultiplier':
                    Action.TAGame_Car_TA_AddedBallForceMultiplier = i
                case 'TAGame.Team_Soccar_TA:GameScore':
                    Action.TAGame_Team_Soccar_TA_GameScore = i
                case 'TAGame.RBActor_TA:bIgnoreSyncing':
                    Action.TAGame_RBActor_TA_bIgnoreSyncing = i
                case 'TAGame.RBActor_TA:WeldedInfo':
                    Action.TAGame_RBActor_TA_WeldedInfo = i
                case 'TAGame.SpecialPickup_BallVelcro_TA:AttachTime':
                    Action.TAGame_SpecialPickup_BallVelcro_TA_AttachTime = i
                case 'TAGame.SpecialPickup_BallVelcro_TA:bHit':
                    Action.TAGame_SpecialPickup_BallVelcro_TA_bHit = i
                case 'TAGame.SpecialPickup_BallVelcro_TA:BreakTime':
                    Action.TAGame_SpecialPickup_BallVelcro_TA_BreakTime = i
                case 'TAGame.SpecialPickup_BallVelcro_TA:bBroken':
                    Action.TAGame_SpecialPickup_BallVelcro_TA_bBroken = i
                case 'TAGame.Car_TA:ReplicatedDemolish_CustomFX':
                    Action.TAGame_Car_TA_ReplicatedDemolish_CustomFX = i
                case 'TAGame.Car_TA:ReplicatedDemolish':
                    Action.TAGame_Car_TA_ReplicatedDemolish = i


    @staticmethod
    def reset():

        Action.Engine_Actor_RemoteRole = -1
        Action.Engine_Actor_DrawScale = -1
        Action.TAGame_CarComponent_TA_Vehicle = -1
        Action.TAGame_CarComponent_TA_ReplicatedActive = -1
        Action.TAGame_CarComponent_Boost_TA_ReplicatedBoostAmount = -1
        Action.TAGame_GameEvent_TA_ReplicatedRoundCountDownNumber = -1
        Action.TAGame_GameEvent_TA_ReplicatedGameStateTimeRemaining = -1
        Action.TAGame_GameEvent_TA_ReplicatedStateName = -1
        Action.TAGame_GameEvent_TA_BotSkill = -1
        Action.TAGame_GameEvent_TA_bHasLeaveMatchPenalty = -1
        Action.TAGame_GameEvent_Team_TA_MaxTeamSize = -1
        Action.TAGame_GameEvent_Soccar_TA_RoundNum = -1
        Action.TAGame_GameEvent_Soccar_TA_bBallHasBeenHit = -1
        Action.TAGame_GameEvent_Soccar_TA_SecondsRemaining = -1
        Action.Engine_Pawn_PlayerReplicationInfo = -1
        Action.TAGame_RBActor_TA_ReplicatedRBState = -1
        Action.TAGame_Vehicle_TA_ReplicatedSteer = -1
        Action.TAGame_Vehicle_TA_ReplicatedThrottle = -1
        Action.TAGame_Vehicle_TA_bReplicatedHandbrake = -1
        Action.TAGame_Vehicle_TA_bDriving = -1
        Action.TAGame_Car_TA_RumblePickups = -1
        Action.TAGame_Car_TA_TeamPaint = -1
        Action.TAGame_CameraSettingsActor_TA_CameraYaw = -1
        Action.TAGame_CameraSettingsActor_TA_CameraPitch = -1
        Action.TAGame_CameraSettingsActor_TA_bMouseCameraToggleEnabled = -1
        Action.TAGame_CameraSettingsActor_TA_bUsingSwivel = -1
        Action.TAGame_CameraSettingsActor_TA_bUsingBehindView = -1
        Action.TAGame_CameraSettingsActor_TA_ProfileSettings = -1
        Action.TAGame_CameraSettingsActor_TA_bUsingSecondaryCamera = -1
        Action.Engine_PlayerReplicationInfo_UniqueId = -1
        Action.Engine_PlayerReplicationInfo_Team = -1
        Action.Engine_PlayerReplicationInfo_PlayerID = -1
        Action.Engine_PlayerReplicationInfo_PlayerName = -1
        Action.Engine_PlayerReplicationInfo_Ping = -1
        Action.TAGame_PRI_TA_CurrentVoiceRoom = -1
        Action.TAGame_PRI_TA_SpectatorShortcut = -1
        Action.TAGame_PRI_TA_SteeringSensitivity = -1
        Action.TAGame_PRI_TA_Title = -1
        Action.TAGame_PRI_TA_PartyLeader = -1
        Action.TAGame_PRI_TA_ClientLoadoutsOnline = -1
        Action.TAGame_PRI_TA_ClientLoadouts = -1
        Action.TAGame_PRI_TA_ReplicatedGameEvent = -1
        Action.TAGame_PRI_TA_PlayerHistoryValid = -1
        Action.TAGame_PRI_TA_MatchScore = -1
        Action.TAGame_Ball_TA_HitTeamNum = -1
        Action.TAGame_CarComponent_Dodge_TA_DodgeTorque = -1
        Action.Engine_GameReplicationInfo_ServerName = -1
        Action.ProjectX_GRI_X_MatchGuid = -1
        Action.ProjectX_GRI_X_bGameStarted = -1
        Action.ProjectX_GRI_X_GameServerID = -1
        Action.ProjectX_GRI_X_Reservations = -1
        Action.ProjectX_GRI_X_ReplicatedServerRegion = -1
        Action.ProjectX_GRI_X_ReplicatedGamePlaylist = -1
        Action.TAGame_Team_TA_GameEvent = -1
        Action.TAGame_VehiclePickup_TA_NewReplicatedPickupData = -1
        Action.TAGame_PRI_TA_PersistentCamera = -1
        Action.TAGame_CameraSettingsActor_TA_PRI = -1
        Action.TAGame_Ball_TA_GameEvent = -1
        Action.TAGame_PRI_TA_MatchGoals = -1
        Action.TAGame_PRI_TA_MatchAssists = -1
        Action.TAGame_PRI_TA_MatchSaves = -1
        Action.TAGame_PRI_TA_MatchShots = -1
        Action.Engine_PlayerReplicationInfo_Score = -1
        Action.Engine_TeamInfo_Score = -1
        Action.TAGame_GameEvent_TA_bCanVoteToForfeit = -1
        Action.TAGame_Car_TA_ReplicatedDemolishGoalExplosion = -1
        Action.TAGame_GameEvent_Soccar_TA_ReplicatedScoredOnTeam = -1
        Action.Engine_Actor_bHidden = -1
        Action.TAGame_PRI_TA_bReady = -1
        Action.TAGame_PRI_TA_ReplicatedWorstNetQualityBeyondLatency = -1
        Action.TAGame_CarComponent_FlipCar_TA_FlipCarTime = -1
        Action.TAGame_GameEvent_Soccar_TA_MaxScore = -1
        Action.TAGame_Team_TA_ClubColors = -1
        Action.TAGame_Car_TA_ClubColors = -1
        Action.ProjectX_GRI_X_ReplicatedGameMutatorIndex = -1
        Action.TAGame_CarComponent_FlipCar_TA_bFlipRight = -1
        Action.TAGame_PRI_TA_bIsDistracted = -1
        Action.TAGame_Team_TA_Difficulty = -1
        Action.TAGame_Team_TA_CustomTeamName = -1
        Action.ProjectX_GRI_X_MatchGUID = -1
        Action.TAGame_GameEvent_Soccar_TA_SeriesLength = -1
        Action.TAGame_CarComponent_Dodge_TA_DodgeImpulse = -1
        Action.Engine_PlayerReplicationInfo_RemoteUserData = -1
        Action.TAGame_GameEvent_Soccar_TA_bOverTime = -1
        Action.TAGame_GameEvent_Soccar_TA_bClubMatch = -1
        Action.TAGame_Team_TA_ClubID = -1
        Action.Engine_PlayerReplicationInfo_bBot = -1
        Action.TAGame_PRI_TA_BotProductName = -1
        Action.TAGame_PRI_TA_ClubID = -1
        Action.TAGame_GameEvent_Team_TA_bForfeit = -1
        Action.TAGame_PRI_TA_PlayerHistoryKey = -1
        Action.Engine_PlayerReplicationInfo_bTimedOut = -1
        Action.Engine_ReplicatedActor_ORS_ReplicatedOwner = -1
        Action.TAGame_MaxTimeWarningData_TA_EndGameWarningEpochTime = -1
        Action.TAGame_MaxTimeWarningData_TA_EndGameEpochTime = -1
        Action.TAGame_Ball_TA_ReplicatedWorldBounceScale = -1
        Action.TAGame_PRI_TA_bIsInSplitScreen = -1
        Action.TAGame_RBActor_TA_bReplayActor = -1
        Action.TAGame_GameEvent_Soccar_TA_bUnlimitedTime = -1
        Action.TAGame_VehiclePickup_TA_bNoPickup = -1
        Action.TAGame_CarComponent_Boost_TA_UnlimitedBoostRefCount = -1
        Action.TAGame_PRI_TA_PawnType = -1
        Action.TAGame_CarComponent_Boost_TA_RechargeDelay = -1
        Action.TAGame_CarComponent_Boost_TA_RechargeRate = -1
        Action.TAGame_BreakOutActor_Platform_TA_DamageState = -1
        Action.TAGame_Ball_Breakout_TA_LastTeamTouch = -1
        Action.TAGame_PRI_TA_MatchBreakoutDamage = -1
        Action.TAGame_Ball_Breakout_TA_AppliedDamage = -1
        Action.TAGame_Ball_Breakout_TA_DamageIndex = -1
        Action.TAGame_PRI_TA_bUsingItems = -1
        Action.TAGame_RumblePickups_TA_ConcurrentItemCount = -1
        Action.TAGame_RumblePickups_TA_PickupInfo = -1
        Action.TAGame_CarComponent_TA_ReplicatedActivityTime = -1
        Action.TAGame_SpecialPickup_Targeted_TA_Targeted = -1
        Action.TAGame_SpecialPickup_BallFreeze_TA_RepOrigSpeed = -1
        Action.TAGame_Car_TA_AddedCarForceMultiplier = -1
        Action.TAGame_Car_TA_AddedBallForceMultiplier = -1
        Action.TAGame_Car_TA_AddedBallForceMultiplier = -1
        Action.TAGame_Team_Soccar_TA_GameScore = -1
        Action.TAGame_RBActor_TA_bIgnoreSyncing = -1
        Action.TAGame_RBActor_TA_WeldedInfo = -1
        Action.TAGame_SpecialPickup_BallVelcro_TA_AttachTime = -1
        Action.TAGame_SpecialPickup_BallVelcro_TA_bHit = -1
        Action.TAGame_SpecialPickup_BallVelcro_TA_BreakTime = -1
        Action.TAGame_SpecialPickup_BallVelcro_TA_bBroken = -1
        Action.TAGame_Car_TA_ReplicatedDemolish_CustomFX = -1
        Action.TAGame_Car_TA_ReplicatedDemolish = -1

        Action.Archetypes_Car_Car_Default = -1 # car
        Action.Archetypes_Ball_Ball_Default = -1 # ball
        Action.TAGame_Default__PRI_TA = -1 # player
        Action.Archetypes_GameEvent_GameEvent_Hockey = -1 # hockey puck
        Action.Archetypes_GameEvent_GameEvent_Basketball = -1 # basketball (hoops ball)
        Action.TAGame_GameEvent_TA_MatchTypeClass = -1 # weird event
        Action.Engine_GameReplicationInfo_GameClass = -1 # weird event
        Action.TAGame_GameEvent_Soccar_TA_ReplicatedStatEvent = -1 # weird event
        Action.TAGame_GameEvent_Soccar_TA_SubRulesArchetype = -1 # weird event
